;-----   Application avec un PIC : Une serrure autonome  tlcarte avec auto-apprentissage ------

; Titre : serrure autonome
; Date : 21 Juillet 2001
; Auteur : P.M
; PIC utilis : PIC 16 F 84
; Ce montage d'initiation  base de PIC 16C84 permet de raliser une serrure  tlcarte
; cette serrure est  auto-apprentissage celle-ci mmorise le contenu d'une tlcarte
; seule une tlcarte identique peut actionner le relais
; une carte de lavage BP peut galement servir de clef

; +++++++++++++ Options du logiciel : ++++++++++++++++++

; 1- une option par switch permet de ne tester que les 12 derniers octets afin d' utiliser 
; plusieurs cartes.
; 2- une option par switch permet soit de temporiser la sortie 7s ds la validation ou bien
; la sortie reste active tant que la tlcarte est prsente, une fois la tlcarte enleve on 
; retrouve la tempo de 7s.
; 3- une option par switch permet de passer en tempo longue soit 25s au lieu des 7 s.

;------------   Directive d' assemblage pour PLAB   ---------------

	list	p=16f84A
	#include p16f84A.inc
	__config H'3FF9'   

;------------   Dfinition des constantes   ---------------

W      			     EQU     0x00        ; variable W = 0 
F	 		     EQU     0x01        ; variable F = 1
					
;------------   Dfinition des registres temporaires   ---------------

retard1      EQU        0x0C      ; le registre temporaire retard1 pour la temporisation
retard2      EQU        0x0D      ; le registre temporaire retard2 pour la temporisation
retard3      EQU        0x19      ; le registre temporaire retard3 pour la temporisation 20s
memo	     EQU	0x14	  ; variable contenant le contenu de la valeur  mmoriser en EPROM
octet	     EQU	0x17	  ; variable dfinissant l 'octet
EEPROM	     EQU	0x16	  ; variable contenant l 'adresse courante de l' EPROM
result	     EQU	0x18	  ; variable contenant une valeur intermdiaire
temp	     EQU	0x20	  ; registre pour divers calculs temporaires
piege	     EQU	0x21	  ; registre qui mmorise une mauvaise carte

;***************   Numero de version  ***************

	org h'2110'
	de "Serrure a autoapprentissage ver 1.0 01/08/01 "

;***************   Programme principal  ***************

	ORG 0
	goto init

;********* Le programme d' interruption se dclenche lorsque 
;          l' entre RB0 ( prsence carte ) est active
	
	 org 4

;********************** INIT et Lecture de la tlcarte ***************************************

	clrf memo		
	clrf octet		
	clrf EEPROM		
	clrf result	
	clrf PORTA
	clrf piege
	
        MOVLW  0x01      	
	addwf octet,F		; on met le registre octet  1 
	addwf temp,F		; on met le registre temp  1 
	addwf piege,F		; on met le registre piege  1 


        MOVLW  0x80		; on passe le registre result  128
	addwf result,F		     	

;+++++++ init de la tlcarte

	clrf PORTB	
	call tempo	
	call tempo
	bsf PORTB,2		; mise  1 du bit pb2 ( RAZ de la tlcarte )	
	goto premier

relecture
	bcf PORTB,1		; raz du bit 1 ( H de la tlcarte ) 
	call tempo	    

premier	

;+++++++ mmorisation du bit adress de la tlcarte

        BTFSc PORTA,0		; on teste le bit RA0 ( S de la tlcarte ) si = 0 on saute 
				; la prochaine instruction
	call addition_1	 

	bcf STATUS,0		; on remet  0 la carry
	rrf result		; on divise result par 2

	bsf PORTB,1		; mise  1 du bit pb1 ( H de la tlcarte )
	call tempo	  
	
 	
	MOVF  octet, W          ; on met octet dans W
	SUBLW 0x08	        ; W - 08(hxa) -> W ( on a fait 8 impulsions ? )
	BTFSC STATUS,2		; on teste le bit Z si Z=0 on saute la prochaine instruction	
	call memo_eprom		; on va au sous programme qui sauve le registre memo en EPROM


        MOVLW  0x01      	; on met 1 dans le registre W
        ADDWF octet, F          ; on additionne octet + 1 ( compteur pour un octet )      


	MOVF  EEPROM, W         ; on met eeprom dans W
	SUBLW 0x20 	        ; W - 20(hxa) -> W ( on a sauvegarder 32 octets ? )
	BTFSC STATUS,2		; on teste le bit Z si Z=0 on saute la prochaine instruction	
	goto fin_lect		; on va au sous programme de fin d'enregistrement fin_lect



	MOVF  piege, W          ; on met le registre piege dans W
	SUBLW 0x01	        ; W - 1 -> W ( mauvaise carte ? )
	BTFSs STATUS,2		; on teste le bit Z si Z=0 on saute la prochaine instruction	
	goto retour_sp		; retour d 'interruption

        goto relecture

addition_1
	movf result,w
	addwf memo,f
	return

;********************** Mmorisation en EPROM ***************************************

memo_eprom

          BTFSs PORTB,7		; on teste le bit RB7 ( mode apprentissage ) si = 1 on saute la prochaine 	
	  goto comparer		; on va au sous programme qui compare le registre memo et l 'octet lu

	  bsf PORTB,5		; mise  1 du bit pb5 (led apprentissage en cours)

 	  bsf STATUS,5		; on met  1 le 5eme bit du registre status pour accder
    				;  la 2eme page mmoire ( pour EECON1 et EECON2 )
          MOVLW 0x04            ; on met 04 dans le registre W
          MOVWF EECON1          ; on met 04 dans le registre EECON1
	  
	  bcf STATUS,5		; on remet  0 le 5eme bit du registre status  

          movf EEPROM,W	        ; on met le registre EEPROM dans W
	  MOVWF EEADR           ; on met W sur EEADR ( adresse EPROM )
          movf memo,W		; on met le contenu du registre memo dans W
	  MOVWF  EEDATA         ; on met W sur EEDATA ( donne EPROM )

   	  bsf STATUS,5		; on met  1 le 5eme bit du registre status pour accder 
    				;  la 2eme page mmoire ( pour EECON1 et EECON2 )
	  
	  movlw 0x55		; passage obligatoire pour ecrire en EPROM
	  movwf EECON2		;...
	  movlw 0xAA		;...
	  movwf EECON2		;...
	  bsf EECON1,1		;...

	  bcf STATUS,5		; on remet  0 le 5eme bit du registre status 

	  
	  MOVLW  0x01           ; on met 1 dans le registre W
          ADDWF EEPROM, F       ; on additionne EEPROM + 1       
	
	  CLRF memo
	  clrf octet
  	  MOVLW  0x80     	
	  addwf result,F	; on met le registre result  128
	  
	  call tempo
	  
	  return

;********************** Lecture EPROM et comparaison avec l'octet lu ***************************************
 
comparer

	 MOVF  EEPROM, W       ; on met le registre EEPROM dans W
         MOVWF  EEADR          ; on met W sur EEADR ( adresse EPROM )
	 bsf STATUS,5	       ; on met  1 le 5eme bit du registre status pour accder
    			       ;  la 2eme page mmoire ( EECON1 )

         MOVLW 0x01            ; on met 01 dans le registre W
         MOVWF EECON1          ; on met 01 dans le registre EECON1
 	 bcf STATUS,5	       ; on remet  0 le 5eme bit du registre status 
	
	 MOVLW  0x01           ; on met 1 dans le registre W
         ADDWF EEPROM, F       ; on additionne EEPROM + 1       
	
	
	 MOVF  EEPROM, W        ; on met octet dans W
	 SUBLW 0x14	        ; W - 14(hxa) -> W ( on a atteint led 12 derniers octets )
	 BTFSC STATUS,2		; on teste le bit Z si Z=0 on saute la prochaine instruction	
	 clrf temp        	; si on a atteint le 12e octet alors temp=0
	 
	 MOVF  EEDATA, W       ; on met la donne EPROM dans W

	 SUBwf memo	       ; W - memo -> W si w=0 alors on continue sinon retour de s-p
	 BTFSs STATUS,2	       ; on teste le bit Z si Z=1  on saute la prochaine instruction	
	 
	 goto mauvaise	       ; si le contenu memo est diff du contenu de l 'EPROM alors on 
                               ; va  mauvaise puis retour de sous-programme d'interruption.		       		
continu
	 CLRF memo
	 clrf octet
  	 MOVLW  0x80     	
	 addwf result,F		; on met le registre result  128
	  
	 call tempo
         return

;********************* Commande du relais ***********************************

relais

;+++++++++ carte OK +++++

	bsf PORTB,6		; mise  1 du bit pb6 ( commande du relais )
	BTFSc PORTB,3		; on teste le bit RB3 ( switch N1 mode impulsion ou mode continu ) 
	goto suivant		; si switch=1 alors on va  suivant
enleve
	BTFSc PORTB,0		; on teste le bit RB0 ( "Prsence carte" ) 
	goto enleve		; on boucle tant que la tlcarte est prsente
	
suivant
	goto tempo2
	
;********************* Mauvaise carte introduite *****************************

mauvaise

	BTFSc PORTB,4		; on teste le bit RB4 ( switch N2 mode 1 carte ou plusieurs ) 
	goto verif		; le s-p verif vrifie que l 'on est dans les 12 derniers octets		
allume
	bsf PORTA,2		; mise  1 du bit pa2 ( led rouge ou bip )
enleve3
	BTFSc PORTB,0		; on teste le bit RB0 ( "Prsence carte" ) 
	goto enleve3		; on boucle tant que la tlcarte est prsente

	clrf piege		; le registre piege est  zro si la carte est mauvaise
	return

;***************** Vrification de la zone des 12 derniers octets**************

; il faut que le registre EEPROM soit > 20 ( dcimal ) pour tre dans la zone des
; 12 derniers octets.

verif

	MOVF  temp, W         ; on met la valeur du registre temp dans W
	SUBLW 0x01	      ; W - 01 -> W 
	BTFSC STATUS,2	      ; on teste le bit Z si Z=0 on saute la prochaine instruction	
	goto continu	      ; on continu la lecture et la comparaison car nous
			      ; ne sommes pas dans la zone des 12 derniers octets
	goto allume

;********************** Temporisation courte T1 ******************************

tempo
	MOVLW 0x25               ; on met 25 dans le registre W
	MOVWF retard1            ; on met W dans le registre retard1 
	MOVWF retard2            ; on met W dans le registre retard2 

attente

	DECFSZ retard1,F         ; on dcrmente retard1 et on saute la prochaine instruction si 
	GOTO  attente            ; le registre retard1 = 0 sinon retour  tempo

	movlw 0x1f		 ; on recharge retard1 avec 1f
	movwf retard1

	DECFSZ retard2,F         ; on dcrmente retard2 et on saute la prochaine instruction si 
	GOTO  attente            ; le registre retard2 = 0 sinon retour  tempo
	 
	RETURN                   ; retour au programme principal aprs l 'instruction CALL

;********************** Temporisation longue T2 20s *******************************

tempo2
	MOVLW 0xff               ; on met ff dans le registre W
	MOVWF retard1            ; on met W dans le registre retard1 
	MOVWF retard2            ; on met W dans le registre retard2 
	
	MOVLW 0x20               ; on met 10 dans le registre W
	MOVWF retard3            ; on met W dans le registre retard3 ( tempo courte )

	MOVLW 0x80               ; on met 80 dans le registre W
	BTFSc PORTA,3		 ; on teste le bit RA3 ( "Switch tempo longue" ) 
	MOVWF retard3            ; on met W dans le registre retard3  ( tempo longue )

attente2

	DECFSZ retard1,F         ; on dcrmente retard1 et on saute la prochaine instruction si 
	GOTO  attente2           ; le registre retard1 = 0 sinon retour  attente2

	movlw 0xff		 ; on recharge retard1
	movwf retard1

	DECFSZ retard2,F         ; on dcrmente retard2 et on saute la prochaine instruction si 
	GOTO  attente2           ; le registre retard2 = 0 sinon retour  attente2

	movlw 0xFF		 ; on recharge retard2
	movwf retard2

	DECFSZ retard3,F         ; on dcrmente retard3 et on saute la prochaine instruction si 
	GOTO  attente2           ; le registre retard3 = 0 sinon retour  attente2
	 
	goto retour_sp

;********* Fin du cycle d'apprentissage ou lecture des 32 octets termine ***********

fin_lect

	BTFSs PORTB,7		; on teste le bit RB7 ( mode apprentissage ) si = 1 on saute la prochaine 	
	goto relais		; instruction sinon on va au sous programme de commande du relais

	bcf PORTB,5		; mise  0 du bit pb5 ( led apprentissage en cours)	

enleve2
	BTFSc PORTB,0		; on teste le bit RB0 ( "Prsence carte" ) 
	goto enleve2		; on boucle tant que la tlcarte est prsente
;||

;********************* Retour de sous programme d 'interruption ********************

retour_sp

	clrf PORTA
	clrf PORTB
	clrf temp
	bcf INTCON,INTF		; on remet  0 le bit du registre d' IT qui est pass  1
	RETFIE			; retour d 'interruption

;********************** Programme d' INIT ***************************************

init
	bsf STATUS,5		; on met  1 le 5eme bit du registre status pour accder
    				;  la 2eme page mmoire ( pour trisa et trisb )
	MOVLW B'10011001'     	; rb0,rb3,rb4 en entre
	MOVWF TRISB           	; on met 00 dans le port B il est programm en sortie

	MOVLW B'11111011'       ; on met f dans le registre W
	MOVWF TRISA           	; on met f dans le port A il est programm en entre pa2 en sortie
	
	bcf STATUS,5		; on remet  0 le 5eme bit du registre status pour accder	
				;  la 1eme page mmoire 
	
	bsf OPTION_REG,INTEDG	; Le passage de 0  1 sur RB0 provoque une IT ( front montant )
	bsf INTCON,INTE		; autorise l ' IT sur RB0
	bcf INTCON,RBIE		; dsactive les IT sur RB4-RB7
	bsf INTCON,GIE		; autorise les Interruptions

	clrf PORTA
	clrf PORTB
	clrf temp

;********************** Programme principal ***************************************

debut 
        sleep 			; mise en sommeil du PIC conso : 3.2 mA, attente RB0 ( prsence carte )
	GOTO debut
	
	END
 






